Param(
    [string] $deploymentConfigsPath,
    [string] $zipArchivePath,
    [string] $mappingConfigsFolderPath,
	[string] $resourceName,
	[bool] $deleteResourceGroupItems,
	[bool] $deployLogicApps
)

# Import utility functions
Import-module ./utils.ps1 -Force

# Read deployment config along with ARM template deployment parameters 
$config = Get-Content -Raw -Path $deploymentConfigsPath | ConvertFrom-Json
$deploymentConfig = $config.deployment;

# Connect to Azure using service principal
.\connect.ps1 -tenantId ${deploymentConfig}.tenantId -subId ${deploymentConfig}.subId -user ${deploymentConfig}.user -pass ${deploymentConfig}.pass

if ($deleteResourceGroupItems)
{
	# Remove all existing resources in the target resource group and deploy main AZ resources from scratch
	Write-Host "WARNING: The action you are about to take will delete all resources under the '$(${deploymentConfig}.rgName)' resource group. Do you wish to continue?" -ForegroundColor Yellow -NoNewline
	$reply = Read-Host -Prompt " (Y/N)"
	if ( $reply -notmatch "[yY]" ) { 
		Write-Host "Deployment process has been stopped"
		return $null
	}

	.\deploy-template.ps1 -stageName ${deploymentConfig}.stageName -rgName ${deploymentConfig}.rgName -templateFile "./templates/delete.$resourceName.stage.arm.json" -templateParametersString "{}" -complete $true
	Start-Sleep -s 5
}

$result = .\deploy-template.ps1 -stageName ${deploymentConfig}.stageName -rgName ${deploymentConfig}.rgName -templateFile "./templates/$resourceName.stage.arm.json" -templateParameters ${config}.templateParameters -complete $deleteResourceGroupItems

if ($null -eq $result)
{
    return;
}

# Read output parameters from the previous ARM deployment (output params could be found in 'outputs' section of the ARM template)
$outputs = $result.Outputs;
$funcAppName = $outputs["funcAppName"].Value
$stMappingsAccountName = $outputs["stMappingsAccountName"].Value
$mappingsContainerName = $outputs["mappingsContainerName"].Value

# Publish zip file with Azure function app code to the target function app
Publish-AzWebapp -ResourceGroupName ${deploymentConfig}.rgName -Name $funcAppName -ArchivePath $zipArchivePath -Force

# Upload all json mapping config files to the target storage account container
$account = Get-AzStorageAccount -ResourceGroupName ${deploymentConfig}.rgName -Name $stMappingsAccountName
Get-ChildItem $mappingConfigsFolderPath | ? {$_.Extension -eq ".json"}  | % {
    Set-AzStorageBlobContent -File $_.FullName -Container $mappingsContainerName -Blob $_.Name -Context ${account}.Context -Force
}

if ($deployLogicApps)
{
	# Deploys logic apps after function app functions became available (preferably replace with warmup logic)
	$attempts = 0;
	do {
		Start-Sleep -s 5
		$logicAppsTemplateParameters = [PSCustomObject]@{ stageName = ${deploymentConfig}.stageName }
		$result = .\deploy-template.ps1 -stageName ${deploymentConfig}.stageName -rgName ${deploymentConfig}.rgName -templateFile "./templates/logic.apps.$resourceName.arm.json" -templateParameters $logicAppsTemplateParameters
		$attempts++;
	} while ($null -ne $result -and $result.ProvisioningState -eq "Failed" -and $attempts -le 2)


	# Read and format logic app callback urls for further use in Cloud Flow connectors
	if ($null -ne $result)
	{
		$callbacks = @{}
		$result.Outputs.Keys | % { $callBacks[$_] = $result.Outputs[$_].Value }
		return $callbacks;
	}

	return $null;
}

